@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/imagefile/imagefile_impl_talon.h>
+#include "imagefile_impl_talon.h"
 #include <mobius/io/file.h>
 #include <mobius/regex_impl.h>
 #include <mobius/string_functions.h>
@@ -67,55 +67,16 @@ imagefile_impl_talon::is_instance (const std::string& url)
 //! \param url imagefile URL
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 imagefile_impl_talon::imagefile_impl_talon (const std::string& url)
-  : imagefile_impl_base (url),
+  : url_ (url),
     split_imagefile_ (get_first_segment_url (url))
 {
-  mobius::io::file f (url);
-  constexpr mobius::io::file::size_type LOG_MAX_SIZE = 65536;
-
-  if (f.exists () && f.get_size () < LOG_MAX_SIZE)
-    {
-      mobius::regex REGEX_TALON_SERIAL ("Serial No.: ?([0-9]+)");
-      mobius::regex REGEX_TALON_SOFTWARE ("Software: ([A-Z0-9.]+)");  
-      mobius::regex REGEX_DRIVE_MODEL ("Drive Model: ([^*]+?) *\\*"); 
-      mobius::regex REGEX_DRIVE_SERIAL ("Serial: ([0-9A-Z-]+) *\\*"); 
-      mobius::regex REGEX_TOTAL_SECTORS ("Drive Size *\\*\r\n\\* *[0-9]+ *[0-9]+ *[0-9]+ *([0-9]+)");      
-
-      // parse .log file
-      auto reader = f.new_reader ();
-      mobius::bytearray data = reader.read (f.get_size ());
-      const std::string text = conv_charset_to_utf8 (data, "ASCII");
-      
-      if (REGEX_TALON_SERIAL.match (text))
-        set_acquisition_platform ("Logicube Talon (s/n: " + REGEX_TALON_SERIAL[1] + ')');
-      
-      if (REGEX_TALON_SOFTWARE.match (text))
-        set_acquisition_tool ("Talon software " + REGEX_TALON_SOFTWARE[1]);
-
-      if (REGEX_DRIVE_MODEL.match (text))
-        set_drive_model (mobius::string::strip (REGEX_DRIVE_MODEL[1]));
-
-      if (REGEX_DRIVE_SERIAL.match (text))
-        set_drive_serial_number (REGEX_DRIVE_SERIAL[1]);
-
-      if (REGEX_TOTAL_SECTORS.match (text))
-        {
-          set_sectors (stoll (REGEX_TOTAL_SECTORS[1]));
-          set_size (get_sectors () * get_sector_size ());
-        }
-        
-      set_segments (split_imagefile_.get_segments ());
-      set_segment_size (split_imagefile_.get_segment_size ());
-      set_acquisition_user (f.get_user_name ());
-      set_acquisition_time (f.get_last_modification_time ());
-    }
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief create new reader for imagefile
 //! \return reader
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const mobius::io::reader
+mobius::io::reader
 imagefile_impl_talon::new_reader () const
 {
   return split_imagefile_.new_reader ();
@@ -125,11 +86,163 @@ imagefile_impl_talon::new_reader () const
 //! \brief create new writer for imagefile
 //! \return writer
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const mobius::io::writer
+mobius::io::writer
 imagefile_impl_talon::new_writer () const
 {
   throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("writer not implemented"));
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get metadata
+//! \return imagefile metadata
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::metadata
+imagefile_impl_talon::get_metadata () const
+{
+  return mobius::metadata
+  {
+    {
+      "url",
+      "URL",
+      "std::string",
+      get_url ()
+    },
+    {
+      "type",
+      "type",
+      "std::string",
+      get_type ()
+    },
+    {
+      "size",
+      "size",
+      "size_type",
+      std::to_string (get_size ()) + " bytes"
+    },
+    {
+      "sectors",
+      "number of sectors",
+      "size_type",
+      std::to_string (get_sectors ())
+    },
+    {
+      "sector_size",
+      "sector size",
+      "size_type",
+      std::to_string (get_sector_size ()) + " bytes"
+    },
+    {
+      "segments",
+      "segments",
+      "size_type",
+      std::to_string (get_segments ())
+    },
+    {
+      "segment_size",
+      "segment_size",
+      "size_type",
+      std::to_string (get_segment_size ()) + " bytes"
+    },
+    {
+      "drive_vendor",
+      "drive vendor",
+      "std::string",
+      get_drive_vendor ()
+    },
+    {
+      "drive_model",
+      "drive model",
+      "std::string",
+      get_drive_model ()
+    },
+    {
+      "drive_serial_number",
+      "drive serial number",
+      "std::string",
+      get_drive_serial_number ()
+    },
+    {
+      "acquisition_user",
+      "acquisition user name",
+      "std::string",
+      get_acquisition_user ()
+    },
+    {
+      "acquisition_time",
+      "acquisition date/time",
+      "mobius::datetime::datetime",
+      to_string (get_acquisition_time ())
+    },
+    {
+      "acquisition_tool",
+      "acquisition tool",
+      "std::string",
+      get_acquisition_tool ()
+    },
+    {
+      "acquisition_platform",
+      "acquisition platform",
+      "std::string",
+      get_acquisition_platform ()
+    },
+  };
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief load metadata on demand
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+imagefile_impl_talon::_load_metadata () const
+{
+  if (metadata_loaded_)
+    return;
+
+  // load metadata
+  mobius::io::file f (url_);
+  constexpr mobius::io::file::size_type LOG_MAX_SIZE = 65536;
+
+  if (f.exists () && f.get_size () < LOG_MAX_SIZE)
+    {
+      mobius::regex REGEX_TALON_SERIAL ("Serial No.: ?([0-9]+)");
+      mobius::regex REGEX_TALON_SOFTWARE ("Software: ([A-Z0-9.]+)");
+      mobius::regex REGEX_DRIVE_MODEL ("Drive Model: ([^*]+?) *\\*");
+      mobius::regex REGEX_DRIVE_SERIAL ("Serial: ([0-9A-Z-]+) *\\*");
+      mobius::regex REGEX_TOTAL_SECTORS ("Drive Size *\\*\r\n\\* *[0-9]+ *[0-9]+ *[0-9]+ *([0-9]+)");
+
+      // parse .log file
+      auto reader = f.new_reader ();
+      mobius::bytearray data = reader.read (f.get_size ());
+      const std::string text = conv_charset_to_utf8 (data, "ASCII");
+
+      if (REGEX_TALON_SERIAL.match (text))
+        acquisition_platform_ = "Logicube Talon (s/n: " + REGEX_TALON_SERIAL[1] + ')';
+
+      if (REGEX_TALON_SOFTWARE.match (text))
+        acquisition_tool_ = "Talon software " + REGEX_TALON_SOFTWARE[1];
+
+      if (REGEX_DRIVE_MODEL.match (text))
+        drive_model_ = mobius::string::strip (REGEX_DRIVE_MODEL[1]);
+
+      if (REGEX_DRIVE_SERIAL.match (text))
+        drive_serial_number_ = REGEX_DRIVE_SERIAL[1];
+
+      if (REGEX_TOTAL_SECTORS.match (text))
+        {
+          sectors_ = stoll (REGEX_TOTAL_SECTORS[1]);
+          size_ = sectors_ * sector_size_;
+        }
+
+      segments_ = split_imagefile_.get_segments ();
+      segment_size_ = split_imagefile_.get_segment_size ();
+      acquisition_user_ = f.get_user_name ();
+      acquisition_time_ = f.get_last_modification_time ();
+
+      normalize_drive_info (drive_vendor_, drive_model_, drive_serial_number_);
+    }
+
+  // set metadata loaded
+  metadata_loaded_ = true;
+}
+
 } // namespace imagefile
 } // namespace mobius
